<?php
/**
 * ownCloud
 *
 * @author Tom Needham <tom@owncloud.com>
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\Controller;

use OCA\Files_Drop\Lib\HelperWrapper;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\TemplateResponse;
use OCP\AppFramework\Http\JSONResponse;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\Controller;
use OCA\Files_Drop\Lib\EndpointService;
use OCP\IRequest;
use OCP\Files;
use OCP\Files\Folder;
use OCP\AppFramework\Http\NotFoundResponse;

/**
 * Class PageController
 *
 * @package OCA\files_drop\Controller
 */
class PageController extends Controller {
	/** @var string */
	private $userId;
	/** @var EndpointService */
	private $endpointService;
	/** @var HelperWrapper */
	private $helperWrapper;

	/**
	 * @param string $AppName
	 * @param IRequest $request
	 * @param string $UserId
	 * @param EndpointService $endpointService
	 * @param HelperWrapper $helperWrapper
	 */
	public function __construct($AppName,
								IRequest $request,
								$UserId,
								EndpointService $endpointService,
								HelperWrapper $helperWrapper) {
		parent::__construct($AppName, $request);
		$this->userId = $UserId;
		$this->endpointService = $endpointService;
		$this->helperWrapper = $helperWrapper;
	}

	/**
	 * Shows a page that allows guests to upload folders to users' specified folders
	 *
	 * @param string $token The endpoint token
	 * @return \OCP\AppFramework\Http\TemplateResponse
	 *
	 * @PublicPage
	 * @NoCSRFRequired
	 */
	public function drop($token) {
		try {
			$endpoint = $this->endpointService->getByToken($token);

			$this->helperWrapper->setupFS($endpoint->getUser());

			$params = [];
			$params['user'] = $endpoint->getUser();
			$params['token'] = $endpoint->getToken();
			$params['maxUpload'] = $this->helperWrapper->maxUploadFilesize($endpoint->getRelativePath());
		} catch (\Exception $e) {
			return new NotFoundResponse();
		}
		return new TemplateResponse($this->appName, 'public', $params, 'guest');
	}

	/**
	 * Handles the upload of files
	 *
	 * @param string $token The endpoint token
	 * @param string $relativePath Optional relative path
	 * @return DataResponse|JSONResponse
	 *
	 * @PublicPage
	 */
	public function upload($token, $relativePath = '') {
		try {
			$endpoint = $this->endpointService->getByToken($token);
			if($endpoint->getUser() === '') {
				return new DataResponse(['message' => 'Endpoint not found'], Http::STATUS_NOT_FOUND);
			}

			$file = $this->request->getUploadedFile('file');
			if(empty($file)) {
				return new DataResponse(['message' => 'No file uploaded'], Http::STATUS_UNPROCESSABLE_ENTITY);
			}

			/** @var Folder $folderView */
			$folderView = $endpoint->getFolderView();

			if($relativePath !== '') {
				try {
					if ($folderView->get($relativePath)) {
						$folderView = $folderView->get($relativePath);
					}
				} catch (\Exception $e) {
					$folderView = $folderView->newFolder($relativePath);
				}
			}

			$notExistingName = $folderView->getNonExistingName($file['name']);
			$uploadedFile = $folderView->newFile($notExistingName);
			$uploadedFile->putContent(fopen($file['tmp_name'], 'rb'));
		} catch (\Exception $exception) {
			return new DataResponse(['message' => 'Server error'], Http::STATUS_INTERNAL_SERVER_ERROR);
		}

		// FIXME: Ugly work-around for older IE versions that do not understand JSON properly for file uploads…
		$response = new JSONResponse(['file' => $notExistingName]);
		$response->setHeaders(['Content-Type' => 'text/plain']);
		return $response;
	}

}
