<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\Tests\Unit\Controller;

use OCA\Files_Drop\Controller\EndpointController;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use Test\TestCase;
use OCP\IRequest;
use OCA\Files_Drop\Lib\EndpointService;
use OCP\IConfig;
use OCP\IURLGenerator;
use OCP\Files\Node;
use OCP\Files\IRootFolder;
use OCA\Files_Drop\Lib\HelperWrapper;

/**
 * Class EndpointControllerTest
 *
 * @package OCA\Files_Drop\Tests\Unit\Controller
 */
class EndpointControllerTest extends TestCase {
	/** @var EndpointController */
	private $endpointController;
	/** @var IRequest */
	private $request;
	/** @var EndpointService */
	private $endpointService;
	/** @var IConfig */
	private $config;
	/** @var IURLGenerator */
	private $urlGenerator;
	/** @var Node */
	private $userFolder;
	/** @var string */
	private $userId;
	/** @var string */
	private $appName;
	/** @var IRootFolder */
	private $rootFolder;
	/** @var HelperWrapper */
	private $helperWrapper;

	public function setUp() {
		parent::setUp();

		$this->endpointService = $this->getMockBuilder('\OCA\Files_Drop\Lib\EndpointService')
			->disableOriginalConstructor()->getMock();
		$this->request = $this->getMockBuilder('\OCP\IRequest')
			->disableOriginalConstructor()->getMock();
		$this->config = $this->getMockBuilder('\OCP\IConfig')
			->disableOriginalConstructor()->getMock();
		$this->urlGenerator = $this->getMockBuilder('\OCP\IURLGenerator')
			->disableOriginalConstructor()->getMock();
		$this->userFolder = $this->getMockBuilder('\OCP\Files\Node')
			->disableOriginalConstructor()->getMock();
		$this->rootFolder = $this->getMockBuilder('\OCP\Files\IRootFolder')
			->disableOriginalConstructor()->getMock();
		$this->helperWrapper = $this->getMockBuilder('\OCA\Files_Drop\Lib\HelperWrapper')
			->disableOriginalConstructor()->getMock();
		$this->appName = 'files_drop';
		$this->userId = 'björn.schießle';

		$this->endpointController = new EndpointController(
			$this->appName,
			$this->userId,
			$this->request,
			$this->endpointService,
			$this->config,
			$this->rootFolder,
			$this->helperWrapper
		);
	}

	public function testDeleteWithoutException() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByUser')
			->with('björn.schießle')
			->willReturn($endpoint);
		$this->endpointService
			->expects($this->once())
			->method('delete')
			->with($endpoint);

		$this->assertEquals(new DataResponse(), $this->endpointController->delete());
	}

	public function testDeleteWithException() {
		$this->endpointService
			->expects($this->once())
			->method('getByUser')
			->with('björn.schießle')
			->will($this->throwException(new \Exception()));

		$expected = new DataResponse(['message' => 'EndpointService not found'], Http::STATUS_NOT_FOUND);
		$this->assertEquals($expected, $this->endpointController->delete());
	}

	public function testCreateWithExceptionGetByUser() {
		$this->endpointService
			->expects($this->once())
			->method('getByUser')
			->with('björn.schießle')
			->will($this->throwException(new \Exception()));

		$expected = new DataResponse(['message' => 'Error occurred'], Http::STATUS_INTERNAL_SERVER_ERROR);
		$this->assertEquals($expected, $this->endpointController->create('MyPath'));
	}

	public function testCreateSuccessful() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();
		$folderView = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByUser')
			->with('björn.schießle')
			->will($this->returnValue($endpoint));
		$this->endpointService
			->expects($this->once())
			->method('getUnusedToken')
			->will($this->returnValue(123));
		$this->endpointService
			->expects($this->once())
			->method('save')
			->with($endpoint);
		$this->rootFolder
			->expects($this->once())
			->method('get')
			->with('björn.schießle/files/')
			->will($this->returnValue($folderView));
		$folderView
			->expects($this->once())
			->method('get')
			->with('MyPath')
			->will($this->returnValue($folderView));
		$folderView
			->expects($this->once())
			->method('getId')
			->will($this->returnValue(23));

		$endpoint
			->expects($this->once())
			->method('setFileId')
			->with(23);
		$endpoint
			->expects($this->once())
			->method('setToken')
			->with(123);

		$endpoint
			->expects($this->once())
			->method('getUrl')
			->will($this->returnValue('http://my.cloud/just/someURI'));
		$endpoint
			->expects($this->exactly(3))
			->method('getRelativePath')
			->will($this->returnValue('/var/www/test'));
		$endpoint
			->expects($this->once())
			->method('getToken')
			->will($this->returnValue('123456789'));
		$this->helperWrapper
			->expects($this->once())
			->method('freeSpace')
			->will($this->returnValue(123456789));
		$this->helperWrapper
			->expects($this->once())
			->method('maxUploadFilesize')
			->will($this->returnValue(123456789));

		$expected = new DataResponse([
			'url' => 'http://my.cloud/just/someURI',
			'path' => '/var/www/test',
			'token' => '123456789',
			'freeSpace' => 123456789,
			'maxUpload' => 123456789,
		]);

		$this->assertEquals($expected, $this->endpointController->create('MyPath'));
	}

	public function testCreateException() {
		$folderView = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();

		$this->rootFolder
			->expects($this->once())
			->method('get')
			->with('björn.schießle/files/')
			->will($this->returnValue($folderView));
		$folderView
			->expects($this->once())
			->method('get')
			->with('MyPath')
			->will($this->returnValue($folderView));
		$folderView
			->expects($this->once())
			->method('getId')
			->will($this->throwException(new \Exception()));

		$expected = new DataResponse([
			'message' => 'Couldn\'t find file ID',
		]);

		$this->assertEquals($expected, $this->endpointController->create('MyPath'));
	}

}
