<?php
/**
 * ownCloud
 *
 * @author Arthur Schiwon <blizzz@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_LDAP_Home;

class Storage extends \OC\Files\Storage\Local {
	const HOME_FOLDER_NAME = 'Home';

	//holds the connection to the LDAP user backend
	static protected $ldap = null;

	//holds the ID of the storage
	private $id = null;

	/**
	 * Constructor
	 *
	 * @param array $arguments holding 'user' key with the user ID and 'home'
	 *	key with user path (see self::setup method)
	 * @throws \Exception when the user is not an LDAP user
	 */
	public function __construct($arguments) {
		$this->initiateLDAPAccess();
		if(!$this->isLdapUser($arguments['user'])) {
			throw new \ErrorException('No LDAP User, no Home – not an Error ');
		}
		$home = self::$ldap->getUserSystemHome($arguments['user']);
		if(!$home) {
			throw new \ErrorException(
				'Cannot determine Home from LDAP for user '.$arguments['user']);
		}
		$this->datadir = $home;
		if(substr($this->datadir, -1) !== '/') {
			$this->datadir .= '/';
		}
		$this->id = 'ldaphome::'.$arguments['user'];
	}

	/**
	 * Get the identifier for the storage,
	 * the returned id should be the same for every storage object that is
	 * created with the same parameters and two storage objects with the same id
	 * should refer to two storages that display the same files.
	 *
	 * @return string
	 */
	public function getId() {
		return $this->id;
	}

	/**
	 * initiates the connection to a LDAP user backend
	 * @throws \Exception if no LDAP configurations are available
	 */
	static private function initiateLDAPAccess() {
		if(!is_null(self::$ldap)) {
			return;
		}
		$configPrefixes =
			\OCA\user_ldap\lib\Helper::getServerConfigurationPrefixes(true);
		if(count($configPrefixes) >= 1) {
			self::$ldap  = new LDAPReader($configPrefixes);
		} else {
			throw new \ErrorException('No LDAP Server configured');
		}
	}

	/**
	 * Checks whether the given user is an LDAP user
	 *
	 * @param string $user optional, the internal username
	 * @return boolean
	 */
	static public function isLdapUser($user = null) {
		if(is_null($user)) {
			$user = \OCP\User::getUser();
		}

		if(is_null(self::$ldap)) {
			self::initiateLDAPAccess();
		}

		if(!self::$ldap->userExists($user)) {
			return false;
		}

		return true;
	}

	/**
	 * Mounts the Home if applicable
	 * This method is triggered by OC_Filesystem::post_initMountPoints hook
	 *
	 * @param array $options offering username and user dir path.
	 */
	static public function setup($options) {
		try {
			//Check whether app requirements are fulfilled and setup shall be
			//done for an LDAP user and if so whether a homedir is set in LDAP
			Helper::checkOperability();
			//The class overwriting is required for unit tests
			$class = isset($options['class']) ? $options['class'] : __CLASS__;
			$storage = new $class(array('user' => $options['user']));
		} catch(\ErrorException $e) {
			return;
		}

		\OCP\Util::writeLog(
			'files_ldap_home', 'Mount Home for '.$options['user'],
			\OCP\Util::DEBUG);
		//Convinced! Mount the Home for this user

		$settings = new Settings();
		$mountName = $settings->MountName;

		//Note: we can use $user/files/ here, because the real home is already
		//mounted thereto after initMountPoints hook
		\OC\Files\Filesystem::mount(
			$storage,
			array(
				'user' => $options['user'],
			),
			$options['user'].'/files/'.$mountName.'/'
		);
	}
}
