<?php
/**
 * ownCloud
 *
 * @author Arthur Schiwon <blizzz@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_LDAP_Home\Tests;

\OC_App::loadApp('user_ldap');
\OC_App::loadApp('files_ldap_home');
\OC::$CLASSPATH['OCA\Files_LDAP_Home\Tests\Storage'] =
	'files_ldap_home/tests/storage.php';

class Test_Storage extends \PHPUnit_Framework_TestCase {
	private $homes = array();

	/**
	 * delete created dir from tmp folder
	 */
	public function __destruct() {
		foreach($this->homes as $path) {
			@rmdir($path);
		}
	}

	/**
	 * preparation for the tests
	 */
	public function setUp() {
		\OC\Files\Filesystem::clearMounts();

		//set up Test Filestructure
		\OC\Files\Filesystem::mount($this->getTempStorage(), array(), '/');
		\OC\Files\Filesystem::mount($this->getTempStorage(), array(),
			'/abrakadabralice/files');
		\OC\Files\Filesystem::mount($this->getTempStorage(), array(),
			'/abrakadabrabob/files');
	}

	/**
	 * get a temporary Storage
	 *
	 * @return \OC\Files\Storage\Temporary
	 */
	private function getTempStorage() {
		$storage = new \OC\Files\Storage\Temporary(array());
		$storage->getScanner()->scan('');
		return $storage;
	}

	/**
	 * Tests mounting for LDAP users (to be succesful) and local/other users
	 * (to be not successful)
	 *
	 * @param string $user username, test LDAP user is called "abrakadabralice"
	 * @param bool $isLdapUser whether the user is an LDAP user
	 */
	private function doMountTest($user, $isLdapUser) {
		if($isLdapUser) {
			//create tmp directory, otherwise mount will fail
			//only necessary for LDAP users. Others don't have  it.
			$this->createHomePath($user);
		}

		$view = new \OC\Files\View('/' . $user . '/files');
		//Check whether mountpoint is not an existing folder.
		//This should pass without any issues.
		$this->doPreCheck($view);

		//the real tests
		$this->doMount($user);
		$mountExists = $view->file_exists('/' . Storage::HOME_FOLDER_NAME)
			&& $view->is_dir('/' . Storage::HOME_FOLDER_NAME);

		if($isLdapUser){
			$this->assertTrue($mountExists, 'Could not mount /Home');
		} else {
			$this->assertFalse($mountExists, 'Mounted /Home were not allowed');
		}

	}

	/**
	 * Kicks off a test with an LDAP user
	 */
	public function testMountOnLdapUser() {
		$this->doMountTest('abrakadabralice', true);
	}

	/**
	 * Kicks off a test with a local/non-ldap user
	 */
	public function testMountOnLocalUser() {
		$this->doMountTest('abrakadabrabob', false);
	}

	/**
	 * Mounts the target storage class
	 *
	 * @param string $user username
	 */
	private function doMount($user) {
		Storage::setup(array(
			'user' => $user,
			'class' => '\OCA\Files_LDAP_Home\Tests\Storage'
		));
	}

	/**
	 * checks whether test setup is clean
	 *
	 * @param \OC\Files\View $view
	 */
	private function doPreCheck($view) {
		$isMountFree = $view->file_exists('/' . Storage::HOME_FOLDER_NAME);
		$this->assertFalse($isMountFree, 'Cannot Test mount, /Home exists');
	}


	/**
	 * creates a directory in the temp folder that will be mounted
	 *
	 * @param string $user username
	 */
	private function createHomePath($user) {
		$home = get_temp_dir(). '/' . $user;
		@mkdir($home);
		$this->homes[] = $home;
	}

}

class Storage extends \OCA\Files_LDAP_Home\Storage {
	public function __construct($arguments) {
		self::$ldap = new MockLDAPReader();

		parent::__construct($arguments);
	}
}

/**
 * Simulates the LDAPReader
 */
class MockLDAPReader extends \OCA\Files_LDAP_Home\LDAPReader {
	public function __construct() {
		//overwrite, it ain't do anything
	}

	public function __destruct() {
		//overwrite, it ain't do anything
	}

	/**
	 * Returns a path for the fake user home directory
	 *
	 * @param string $uid username
	 * @return string path like /tmp/userxy
	 */
	public function getUserSystemHome($uid) {
		$home = get_temp_dir(). '/' . $uid;
		return $home;
	}

	/**
	 * Mocks the userExists method, returns true for a defined usermname
	 *
	 * @param string $uid username
	 * @return boolean
	 */
	public function userExists($uid) {
		if($uid == 'abrakadabralice') {
			return true;
		}
		return false;
	}
}
