<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
\OCP\User::checkAdminUser();
\OCP\JSON::callCheck();

$mandatoryParams = array('WNDMountPoint', 'WNDUrl', 'WNDShare', 'WNDRoot', 'WNDCredentialsType');

// global username and password can be empty to connect as guest
if ($_POST['WNDCredentialsType'] !== "1") {
    $_POST['WNDGuser'] = Utils::getDefault($_POST, 'WNDGuser', '');
    $_POST['WNDGpass'] = Utils::getDefault($_POST, 'WNDGpass', '');
}

$l = OC_L10N::get('windows_network_drive');
// check mandatory parameters
foreach ($mandatoryParams as $mparam) {
    if (!isset($_POST[$mparam]) || ($_POST[$mparam] !== "0" && empty($_POST[$mparam]))) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Missing mandatory parameters'))));
        exit();
    }
}

// check special chars in the mount point
Utils::checkInvalidChars($_POST['WNDMountPoint'], '\\/<>:"\'|?*', function($list) use ($l) {
    \OCP\JSON::error(array("data" => array("message" => $l->t('The mount point contains one of the following invalid chars: %s', implode(', ', $list)))));
    exit();
});

// build the applicables based on data
$_POST['WNDMountType'] = Utils::getDefault($_POST, 'WNDMountType', '');
$applicables = array();
if ($_POST['WNDMountType'] === '') {
    $applicables['global'] = 'global';
} else {
    $applicableList = explode(',', $_POST['WNDMountType']);
    foreach ($applicableList as $element) {
        if ($element === 'global') {
            $applicables['global'] = 'global';
        }
        // WARNING: Need to check under special conditions: jjj(user)(group) can cause issues
        $username = substr($element, 0, strrpos($element, '(user)'));
        $groupname = substr($element, 0, strrpos($element, '(group)'));
        if (!empty($username)) {
            if (!isset($applicables['users'])) {
                $applicables['users'] = array();
            }
            $applicables['users'][] = $username;
        }
        if (!empty($groupname)) {
            if (!isset($applicables['groups'])) {
                $applicables['groups'] = array();
            }
            $applicables['groups'][] = $groupname;
        }
    }
}


switch ($_POST['WNDCredentialsType']) {
    case "0":
        // admin provides credentials for this mount point only
        $params = array('host' => $_POST['WNDUrl'],
                        'share' => $_POST['WNDShare'],
                        'root' => $_POST['WNDRoot'],
                        'user' => $_POST['WNDGuser'],
                        'password' => $_POST['WNDGpass']);
        $wnd = new \OCA\windows_network_drive\lib\WND($params);
        try {
            if ($wnd->test()) {
                $encryptedPass = Utils::encryptPassword($_POST['WNDGpass']);
                DBAccess::saveMount($_POST['WNDMountPoint'], $applicables, $_POST['WNDUrl'],
                                    $_POST['WNDShare'], $_POST['WNDRoot'], 0,
                                    $_POST['WNDGuser'], $encryptedPass);
            } else {
                \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                exit();
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
            exit();
        }
        break;

    case "1":
        // users provide their own credentials
        DBAccess::saveMount($_POST['WNDMountPoint'], $applicables, $_POST['WNDUrl'],
                            $_POST['WNDShare'], $_POST['WNDRoot'], 1, null, null);
        break;

    case "2":
        // admin uses its global credentials
        $gcreds = DBAccess::getAdminGlobalCredentialsCached();
        $decryptedPassword = Utils::decryptPassword($gcreds['global_password']);
        if ($decryptedPassword === false) {
            \OC_Log::write('wnd', 'cannot decrypt: ' . $gcreds['global_password'], \OC_Log::ERROR);
        }
        $params = array('host' => $_POST['WNDUrl'],
                        'share' => $_POST['WNDShare'],
                        'root' => $_POST['WNDRoot'],
                        'user' => $gcreds['global_username'],
                        'password' => $decryptedPassword);
        $wnd = new \OCA\windows_network_drive\lib\WND($params);
        try {
            if ($wnd->test()) {
                $encryptedPass = Utils::encryptPassword($_POST['WNDGpass']);
                DBAccess::saveMount($_POST['WNDMountPoint'], $applicables, $_POST['WNDUrl'],
                                    $_POST['WNDShare'], $_POST['WNDRoot'], 2, null, null);
            } else {
                \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                exit();
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
            exit();
        }
        break;

	case '3':
		// The ownCloud login credentials are used
		DBAccess::saveMount($_POST['WNDMountPoint'], $applicables, $_POST['WNDUrl'],
			$_POST['WNDShare'], $_POST['WNDRoot'], 3, null, null);
		break;
}

$id = null;
$cursor = DBAccess::getMountByPropsLast($_POST['WNDMountPoint'], $_POST['WNDUrl'],
                                        $_POST['WNDShare'], $_POST['WNDRoot']);
if ($cursor && ($row = $cursor->fetchRow()) !== false) {
    $id = intval($row['mount_id']);
}
\OCP\JSON::success(array("data" => $l->t('ok'), "id" => $id));
