<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\windows_network_drive\lib;

use \OCA\windows_network_drive\lib\Utils;
use \OCA\windows_network_drive\lib\DBAccess;

/**
 * Utility class to manage WND easily.
 */
class WNDUtils {
    private static function decryptPasswordInParams($fsparams) {
        if (!array_key_exists('password', $fsparams)) {
            \OC_Log::write('wnd', 'missing password mounting the FS', \OC_Log::ERROR);
            return $fsparams;
        }

        if ($fsparams['password'] !== null) {
            $correctPassword = Utils::decryptPassword($fsparams['password']);
            if ($correctPassword === false) {
                \OC_Log::write('wnd', 'cannot decrypt password: ' . $fsparams['password'], \OC_Log::ERROR);
                return $fsparams;
            }

            $fsparams['password'] = $correctPassword;
        }
        return $fsparams;
    }
    private static function mountWndWithParams($user, $rowMountPoint, array $fsparams) {
        $fsparams = self::decryptPasswordInParams($fsparams);
        $fsparams['oc_sharing'] = \OCP\Config::getAppValue('windows_network_drive', 'global_sharing', false);
        $mountPoint = "/$user/files/" . $rowMountPoint;
        \OC\Files\Filesystem::mount('\OCA\windows_network_drive\lib\WND',
                                    $fsparams, $mountPoint);
    }
    /**
     * Mount a global (admin) mount point for the user with the row data
     *
     * @param string $user the user mounting the storage. This user will be used to get the
     * personal credentials if needed, and to mount the storage. $user should match the
     * $row['mounting_user'] parameter
     * @param array $row a row coming from the DB with all the data to mount the storage
     */
    public static function mountGlobalWndRow($user, $row) {
        if (intval($row['use_custom_credentials']) === 0 && !is_null($row['global_username']) && !is_null($row['global_password'])) {
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $row['global_username'],
                                "password" => $row['global_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else if (intval($row['use_custom_credentials']) === 1 && !is_null($row['credential_username']) && !is_null($row['credential_password'])) {
            if (intval($row['use_global_credentials']) === 0) {
                $fsparams = array ("host" => $row['mount_url'],
                                    "user" => $row['credential_username'],
                                    "password" => $row['credential_password'],
                                    "share" => $row['share'],
                                    "root" => $row['root']);
            } else {
                $userGlobalCreds = DBAccess::getPersonalGlobalCredentialsCached($user);
                $fsparams = array ("host" => $row['mount_url'],
                                    "user" => $userGlobalCreds['global_username'],
                                    "password" => $userGlobalCreds['global_password'],
                                    "share" => $row['share'],
                                    "root" => $row['root']);
            }
        } else if (intval($row['use_custom_credentials']) === 2) {
            $globalCreds = DBAccess::getAdminGlobalCredentialsCached();
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $globalCreds['global_username'],
                                "password" => $globalCreds['global_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else if (intval($row['use_custom_credentials']) === 3) {
            $params_auth = json_decode(\OC::$server->getCrypto()->decrypt(\OC::$server->getSession()->get('wnd-credentials')), true);
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $params_auth['uid'],
                                "password" => $params_auth['password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else {
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => null,
                                "password" => null,
                                "share" => $row['share'],
                                "root" => $row['root']);
        }
        self::mountWndWithParams($user, $row['mount_point'], $fsparams);
    }

    /**
     * Mount a personal mount point for the user with the row data
     *
     * @param string $user the user mounting the storage. This user will be used to get the
     * personal credentials if needed, and to mount the storage. $user should match the
     * $row['target_user'] parameter
     * @param array $row a row coming from the DB with all the data to mount the storage
     */
    public static function mountPersonalWndRow($user, $row) {
        if (intval($row['use_global_credentials']) === 0) {
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $row['creds_username'],
                                "password" => $row['creds_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else {
            $userGlobalCreds = DBAccess::getPersonalGlobalCredentialsCached($user);
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $userGlobalCreds['global_username'],
                                "password" => $userGlobalCreds['global_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        }
        self::mountWndWithParams($user, $row['mount_point'], $fsparams);
    }

    /**
     * Get a WND instance according to the data coming from row.
     *
     * @param array $row a row coming from the DB with all the data to mount the global (admin)
     * storage
     * @return \OCA\windows_network_drive\lib\WND The WND instance for the row
     */
    public static function getWndForGlobalRow($row) {
        if (intval($row['use_custom_credentials']) === 0 && !is_null($row['global_username']) && !is_null($row['global_password'])) {
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $row['global_username'],
                                "password" => $row['global_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else if (intval($row['use_custom_credentials']) === 1 && !is_null($row['credential_username']) && !is_null($row['credential_password'])) {
            if (intval($row['use_global_credentials']) === 0) {
                $fsparams = array ("host" => $row['mount_url'],
                                    "user" => $row['credential_username'],
                                    "password" => $row['credential_password'],
                                    "share" => $row['share'],
                                    "root" => $row['root']);
            } else {
                $userGlobalCreds = DBAccess::getPersonalGlobalCredentialsCached($row['mounting_user']);
                $fsparams = array ("host" => $row['mount_url'],
                                    "user" => $userGlobalCreds['global_username'],
                                    "password" => $userGlobalCreds['global_password'],
                                    "share" => $row['share'],
                                    "root" => $row['root']);
            }
        } else if (intval($row['use_custom_credentials']) === 2) {
            $globalCreds = DBAccess::getAdminGlobalCredentialsCached();
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $globalCreds['global_username'],
                                "password" => $globalCreds['global_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else if (intval($row['use_custom_credentials']) === 3) {
            $params_auth = json_decode(\OC::$server->getCrypto()->decrypt(\OC::$server->getSession()->get('wnd-credentials')), true);
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $params_auth['uid'],
                                "password" => $params_auth['password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else {
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => null,
                                "password" => null,
                                "share" => $row['share'],
                                "root" => $row['root']);
        }
        $fsparams = self::decryptPasswordInParams($fsparams);
        if (is_null($fsparams)) {
            return;
        }
        $fsparams['oc_sharing'] = \OCP\Config::getAppValue('windows_network_drive', 'global_sharing', false);
        return new \OCA\windows_network_drive\lib\WND($fsparams);
    }

    /**
     * Get a WND instance according to the data coming from row.
     *
     * @param array $row a row coming from the DB with all the data to mount the personal storage
     * @return \OCA\windows_network_drive\lib\WND The WND instance for the row
     */
    public static function getWndForPersonalRow($row) {
        if (intval($row['use_global_credentials']) === 0) {
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $row['creds_username'],
                                "password" => $row['creds_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        } else {
            $userGlobalCreds = DBAccess::getPersonalGlobalCredentialsCached($row['target_user']);
            $fsparams = array ("host" => $row['mount_url'],
                                "user" => $userGlobalCreds['global_username'],
                                "password" => $userGlobalCreds['global_password'],
                                "share" => $row['share'],
                                "root" => $row['root']);
        }

        $fsparams = self::decryptPasswordInParams($fsparams);
        if (is_null($fsparams)) {
            return;
        }
        $fsparams['oc_sharing'] = \OCP\Config::getAppValue('windows_network_drive', 'global_sharing', false);
        return new \OCA\windows_network_drive\lib\WND($fsparams);
    }

    /**
     * Perform a test for the wnd and get results
     *
     * The result is an array with the following information:
     * - "status" : "ok" or "fail"
     * - "code" : if the status is "ok" this field is missing, otherwise it can have the
     * following values
     *  - "TF" : for "test failed"
     *  - "AD" : for "access denied"
     *  - "IH" : for "invalid host"
     *  - "CE" : for "connection error"
     *  - "GE" : for "generic error"
     * - "error" : if the status is "ok" this field is missing, otherwise a translatable error
     * message explaining the situation is here
     *
     * @param \OCA\windows_network_drive\lib\WND $wnd the WND instance where the test will be
     * performed
     * @param string $mountPoint the mount point for the instance
     * @param array $test an array where the results will be stored. As long as the mount points
     * are different, this array can be reused to store more tests
     * @param \OC_L10N $l, and L10N instance to get the translation
     */
    public static function performTestOverWnd($wnd, $mountPoint, &$test, $l) {
        try {
            $result = $wnd->test();
            if ($result) {
                $test[$mountPoint] = array('status' => 'ok');
            } else {
                $test[$mountPoint] = array('code' => 'TF', 'error' => $l->t('File doesn\'t exists'));
            }
        } catch (\Icewind\SMB\Exception\AccessDeniedException $e) {
            $test[$mountPoint] = array('code' => 'AD', 'error' => $l->t('Access denied to the server'));
        } catch (\Icewind\SMB\Exception\ForbiddenException $e) {
            $test[$mountPoint] = array('code' => 'AD', 'error' => $l->t('Access denied to the server'));
        } catch (\Icewind\SMB\Exception\InvalidHostException $e) {
            $test[$mountPoint] = array('code' => 'IH', 'error' => $l->t('Invalid host'));
        } catch (\Icewind\SMB\Exception\ConnectionError $e) {
            $test[$mountPoint] = array('code' => 'CE', 'error' => $l->t('Cannot connect to the server'));
        } catch (\Exception $e) {
            // handle special cases that don't throw the right exceptions
            $errorCode = $e->getCode();
            switch ($errorCode) {
                case 110:
                case 111:
                case 112:
                case 113:
                    $test[$mountPoint] = array('code' => 'CE', 'error' => $l->t('Cannot connect to the server'));
                    break;
                default:
                    $test[$mountPoint] = array('code' => 'GE', 'error' => $l->t('Unknown error: ' . $e->getMessage()));
            }
        }

        if (!isset($test[$mountPoint]['status'])) {
            $test[$mountPoint]['status'] = 'fail';
        }
    }

    public static function performTestOverGlobalRow($row, &$test, $l) {
        $wnd = WNDUtils::getWndForGlobalRow($row);
        if (is_null($wnd)) {
            $test[$row['mount_point']]['code'] = 'CNP';
            $test[$row['mount_point']]['type'] = 'global';
            $test[$row['mount_point']]['location'] = intval($row['use_custom_credentials']);
            $test[$row['mount_point']]['mid'] = intval($row['mount_id']);
            $test[$row['mount_point']]['url'] = $row['mount_url'];
            $test[$row['mount_point']]['share'] = $row['share'];
            $test[$row['mount_point']]['status'] = 'fail';
            $test[$row['mount_point']]['error'] = $l->t('Cannot decrypt credentials');
        } else {
            WNDUtils::performTestOverWnd($wnd, $row['mount_point'], $test, $l);
            // we'll add more information to the test
            $test[$row['mount_point']]['type'] = 'global';
            $test[$row['mount_point']]['location'] = intval($row['use_custom_credentials']);
            $test[$row['mount_point']]['mid'] = intval($row['mount_id']);
            $test[$row['mount_point']]['url'] = $row['mount_url'];
            $test[$row['mount_point']]['share'] = $row['share'];

            if ($test[$row['mount_point']]['status'] === 'ok' ||
                    !in_array($test[$row['mount_point']]['code'], array('CE', 'GE'))) {
                // if check is ok or the error is not due to not having access to the server
                // we need to check if we have valid credentials, to show the proper message
                if ($row['use_custom_credentials'] === "1" && (is_null($row['credential_username']) || is_null($row['credential_password']))) {
                    $test[$row['mount_point']] = array('status' => 'fail', 'mid' => intval($row['mount_id']),
                                                        'location' => intval($row['use_custom_credentials']),
                                                        'type' => 'global', 'code' => 'CNP',
                                                        'url' => $row['mount_url'],
                                                        'share' => $row['share'],
                                                        'error' => $l->t('Credentials not provided yet'));
                }
            }
        }
    }

    public static function performTestOverPersonalRow($row, &$test, $l) {
        $wnd = WNDUtils::getWndForPersonalRow($row);
        if (is_null($wnd)) {
            $test[$row['mount_point']]['code'] = 'CNP';
            $test[$row['mount_point']]['type'] = 'personal';
            $test[$row['mount_point']]['location'] = intval($row['use_global_credentials']);
            $test[$row['mount_point']]['mid'] = intval($row['personal_mount_id']);
            $test[$row['mount_point']]['url'] = $row['mount_url'];
            $test[$row['mount_point']]['share'] = $row['share'];
            $test[$row['mount_point']]['status'] = 'fail';
            $test[$row['mount_point']]['error'] = $l->t('Cannot decrypt credentials');
        } else {
            WNDUtils::performTestOverWnd($wnd, $row['mount_point'], $test, $l);
            $test[$row['mount_point']]['type'] = 'personal';
            $test[$row['mount_point']]['location'] = intval($row['use_global_credentials']);
            $test[$row['mount_point']]['mid'] = intval($row['personal_mount_id']);
            $test[$row['mount_point']]['url'] = $row['mount_url'];
            $test[$row['mount_point']]['share'] = $row['share'];
        }
    }

    public static function getMountsForApi($params) {
        $entries = array();
        $user = \OC_User::getUser();
        $groups = \OC_Group::getUserGroups($user);
        $mountList = DBAccess::getMountsForUser($user, $groups);
        if ($mountList !== false) {
            while (($row = $mountList->fetchRow()) !== false) {
                $entries[] = self::generateApiEntryForRow($row, 'system');
            }
        }

        $personalMountList = DBAccess::getPersonalMountsPerUser($user);
        if ($personalMountList !== false) {
            while (($row = $personalMountList->fetchRow()) !== false) {
                $entries[] = self::generateApiEntryForRow($row, 'personal');
            }
        }
        return new \OC_OCS_Result($entries);
    }

    private static function generateApiEntryForRow($row, $type) {
        $l = \OC_L10N::get('windows_network_drive');  //TODO: Need to find a better place for this
        $path = dirname($row['mount_point']);
        if ($path === '.') {
            $path = "";
        }
        $map = array('personal' => array(0 => $l->t('Credentials only for this mount'),
                                        2 => $l->t('Personal credentials')),
                    'system' => array(0 => $l->t('Admin-provided credentials'),
                                        1 => $l->t('Per-user credentials'),
                                        2 => $l->t('Admin-provided credentials')));
        $columnKey = null;
        if ($type === 'system') {
            $columnKey = 'use_custom_credentials';
        } else if ($type === 'personal') {
            $columnKey = 'use_global_credentials';
        }

        if ($columnKey === null) {
            $auth = $l->t('Unknown');
        } else {
            $auth = Utils::getDefault($map[$type], $row[$columnKey], $l->t('Unknown'));
        }

        return array(
            'name' => basename($row['mount_point']),
            'server' => $row['mount_url'],
            'share' => $row['share'],
            'auth' => $auth,
            'path' => $path,
            'type' => 'dir',
            'scope' => $type,
            'permissions' => \OCP\PERMISSION_READ
        );
    }
}
